package com.bureau.devicefingerprint.tools

import android.util.Log
import com.bureau.devicefingerprint.security.SecureJNI
import com.google.gson.GsonBuilder
import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec
import kotlin.text.Charsets.UTF_8

object DeviceDataEncryptionUtils {
    private val md5HashValueKey = SecureJNI.getPKP1c()
    private val dummyKey = SecureJNI.getPKP2c()
    private val dummyValue = SecureJNI.getPKP3c()
    private val ivGlobal = SecureJNI.getPKP4c().toByteArray(StandardCharsets.UTF_8)
    private val passPhrase = SecureJNI.getPKP5c().toByteArray(StandardCharsets.UTF_8)

    private const val TAG = "DeviceDataEncryption"

    private fun ByteArray.toHex() = joinToString(separator = "") { byte -> "%02x".format(byte) }

    fun <T> getEncryptedPayload(payLoad: T): String {
        val md5EncryptedPayLoad = getMD5EncryptedPayLoad(payLoad) ?: ""
        return getAESEncryptedPayLoad(md5EncryptedPayLoad)
    }

    private fun <T> getMD5EncryptedPayLoad(payLoad: T): String? {
        val gson = GsonBuilder()
            .serializeNulls()
            .disableHtmlEscaping()
            .create()
        val payloadStringifiedBeforeSortingKeys: String = gson.toJson(payLoad)
        val payLoadTreeMap = JSONUtils.jsonToMap(payloadStringifiedBeforeSortingKeys)
        payLoadTreeMap.put(dummyKey, dummyValue)
        val payloadStringifiedAfterSortingKeys = gson.toJson(payLoadTreeMap)
        val crypt = MessageDigest.getInstance("MD5")
            .digest(payloadStringifiedAfterSortingKeys.toByteArray(UTF_8))
        payLoadTreeMap.put(md5HashValueKey, crypt.toHex())
        payLoadTreeMap.remove(dummyKey)
        Log.d(TAG, "main: $payLoad")
        Log.d(TAG, "main: $payloadStringifiedBeforeSortingKeys")
        Log.d(TAG, "main: $payloadStringifiedAfterSortingKeys")

        val finalMd5EncryptedPayLoad = gson.toJson(payLoadTreeMap)
        Log.d(TAG, "main: $finalMd5EncryptedPayLoad")
        return finalMd5EncryptedPayLoad
    }

    private fun encrypt(key: SecretKeySpec, data: ByteArray): ByteArray {
        val cipher = Cipher.getInstance("AES/CBC/PKCS5Padding")
        cipher.init(Cipher.ENCRYPT_MODE, key, IvParameterSpec(ivGlobal))
        return cipher.doFinal(data)
    }

    private fun arrayBufferToHexString(buffer: ByteArray): String {
        return buffer.joinToString("") { byte -> "%02x".format(byte) }
    }

    private fun getAESEncryptedPayLoad(payLoad: String): String {
        val key = SecretKeySpec(passPhrase, "AES")
        val data = payLoad.toByteArray(StandardCharsets.UTF_8)
        val encrypted = encrypt(key, data)
        val encryptedString = arrayBufferToHexString(encrypted)
        Log.d(TAG, "main2: $encryptedString")
        return encryptedString
    }
}
